@@ -60,24 +60,26 @@
 #' afr = runif(5)
 #' afr = afr/sum(afr)
 #'
-#' pow1 = LRpower(claim, unrel, truth, ids = ids, nsim = nsim, threshold = thresh,
-#'                alleles = als, afreq = afr, seed = 123)
+#' pow1 = LRpower(claim, unrel, truth, ids = ids, nsim = nsim,
+#'                threshold = thresh, alleles = als, afreq = afr,
+#'                seed = 123)
 #' pow1
 #'
 #' # Simulation 2: Same, but using an attached marker
 #' m = marker(truth, alleles = als, afreq = afr)
 #' truth = setMarkers(truth, m)
 #'
-#' pow2 = LRpower(claim, unrel, truth, ids = ids, nsim = nsim, threshold = thresh,
-#'                markers = 1, seed = 123)
+#' pow2 = LRpower(claim, unrel, truth, ids = ids, nsim = nsim,
+#'                threshold = thresh, markers = 1, seed = 123)
+#'
 #' stopifnot(identical(pow1$LRperSim, pow2$LRperSim))
 #'
 #' \donttest{
 #' # Founder inbreeding in true pedigree
 #' founderInbreeding(truth, founders(truth)) = 0.5
 #' truth
-#' pow3 = LRpower(claim, unrel, truth, ids = ids, nsim = nsim, threshold = thresh,
-#'                markers = 1, seed = 123, plot = TRUE)
+#' pow3 = LRpower(claim, unrel, truth, ids = ids, nsim = nsim,
+#'                threshold = thresh, markers = 1, seed = 123, plot = TRUE)
 #' pow3
 #' }
 #'
@@ -87,8 +89,13 @@ LRpower = function(numeratorPed, denominatorPed, truePed = numeratorPed, ids, ma
                    alleles = NULL, afreq = NULL, Xchrom = FALSE, knownGenotypes = NULL,
                    plot = FALSE, plotMarkers = NULL, seed = NULL, verbose = TRUE) {
   st = Sys.time()
-  ids = as.character(ids)
-  source = match.arg(source, c("true", "numerator", "denominator"))
+  if(is.list(ids)) {
+    ids = lapply(ids, as.character)
+    allids = unique.default(unlist(ids))
+  }
+  else {
+    allids = as.character(ids)
+  }
 
   ### Input option 1: Single marker with attributes given directly
   if(!is.null(alleles) || !is.null(afreq)) {
@@ -107,7 +114,7 @@ LRpower = function(numeratorPed, denominatorPed, truePed = numeratorPed, ids, ma
     }
 
     # If `alleles` is single integer, convert to sequence
-    if(length(alleles) == 1 && is.numeric(alleles))
+    if(isNumber(alleles))
       alleles = seq_len(alleles)
 
     # Create and attach locus to both pedigrees
@@ -116,10 +123,10 @@ LRpower = function(numeratorPed, denominatorPed, truePed = numeratorPed, ids, ma
 
     markers = 1
     typed = typedMembers(truePed)
-    #hasMut = FALSE
     disableMutations = FALSE # don't do anything
   }
   else {
+    source = match.arg(source, c("true", "numerator", "denominator"))
     sourcePed = switch(source, true = truePed, numerator = numeratorPed, denominator = denominatorPed,
                        stop2("`source` must be either 'true', 'numerator' or 'denominator': ", source))
 
@@ -138,7 +145,7 @@ LRpower = function(numeratorPed, denominatorPed, truePed = numeratorPed, ids, ma
 
     # Check for already typed members. TODO: Support for partially typed members
     typed = typedMembers(sourcePed)
-    if(length(bad <- intersect(ids, typed)))
+    if(length(bad <- intersect(allids, typed)))
       stop2("Individual is already genotyped: ", toString(bad))
 
     # Select markers from source and transfer to truePed (if neccessary)
@@ -166,9 +173,9 @@ LRpower = function(numeratorPed, denominatorPed, truePed = numeratorPed, ids, ma
       frms = c("Numerator", "Denominator", "True")
     }
 
-    plotPedList(peds, newdev = TRUE, frametitles = frms,
-                shaded = function(p) c(ids, typedMembers(p)),
-                col = list(red = ids),
+    plotPedList(peds, newdev = TRUE, titles = frms,
+                hatched = function(p) c(allids, typedMembers(p)),
+                col = list(red = allids),
                 marker = match(plotMarkers, markers))
 
     if (plot == "plotOnly")
@@ -182,16 +189,49 @@ LRpower = function(numeratorPed, denominatorPed, truePed = numeratorPed, ids, ma
   if(verbose)
     message("Simulating ", nsim, " profiles from true pedigree...", appendLF = FALSE)
 
-  allsims = profileSim(truePed, N = nsim, ids = ids)
+  allsims = profileSim(truePed, N = nsim, ids = allids, verbose = FALSE)
 
   if(verbose)
-    message("done\nComputing likelihood ratios...", appendLF = FALSE)
+    message("done")
 
-  # Compute the exclusion power of each marker
-  lrs = vapply(allsims, function(s) {
-    numerSim = transferMarkers(from = s, to = numeratorPed)
-    denomSim = transferMarkers(from = s, to = denominatorPed)
+  # List of input parameters
+  params = list(markers = markers, nsim = nsim, threshold = threshold, seed = seed,
+                disableMutations = disableMutations, typed = typed, allids = allids)
 
+  if(is.list(ids))
+    res = lapply(ids, function(idvec) lrPowerCompute(allsims, numeratorPed, denominatorPed,
+                                                     idvec, params = params, verbose = verbose))
+  else
+    res = lrPowerCompute(allsims, numeratorPed, denominatorPed, ids,
+                         params = params, verbose = verbose)
+
+  # Timing
+  time = Sys.time() - st
+  if(verbose)
+    message("Total time used: ", format(time, digits = 3))
+
+  # Return results
+  res
+}
+
+
+lrPowerCompute = function(sims, numeratorPed, denominatorPed, ids, params, verbose = TRUE) {
+
+  if(verbose)
+    message("Computing LR distribution for individuals ", toString(ids), "...", appendLF = FALSE)
+
+  markers = params$markers
+  threshold = params$threshold
+
+  # Target individuals: Pre-typed and simulated
+  targetsIds = c(params$typed, ids)
+
+  # Trans
+  # Compute LR distribution using the ids individuals only
+  lrs = vapply(sims, function(s) {
+    numerSim = transferMarkers(from = s, to = numeratorPed, ids = targetsIds)
+    denomSim = transferMarkers(from = s, to = denominatorPed, ids = targetsIds)
+    #print(numerSim);print(denomSim)
     lr = kinshipLR(list(numerSim, denomSim), ref = 2)
     lr$LRperMarker[,1]
   }, FUN.VALUE = numeric(length(markers)))
@@ -213,29 +253,21 @@ LRpower = function(numeratorPed, denominatorPed, truePed = numeratorPed, ids, ma
   IP = sapply(threshold, function(thr) mean(LRperSim >= thr))
   names(IP) = threshold
 
-  # Timing
-  time = Sys.time() - st
-  if(verbose)
-    message("Total time used: ", format(time, digits = 3))
-
-  # Lits of input parameters
-  params = list(ids = ids, markers = markers,
-                nsim = nsim, threshold = threshold, seed = seed,
-                disableMutations = disableMutations)
-
+  params$ids = ids
   structure(list(LRperSim = LRperSim, meanLRperMarker = meanLRperMarker,
                  meanLR = meanLR, meanLogLR = meanLogLR, IP = IP,
-                 time = time, params = params), class = "LRpowerResult")
+                 params = params), class = "LRpowerResult")
+
 }
 
 #' @export
 print.LRpowerResult = function(x, ...) {
-  cat("Mean total LR:", round(x$meanLR, 3), "\n")
-  cat("Mean total log10(LR):", round(x$meanLogLR, 3), "\n")
+  cat("Mean LR:", round(x$meanLR, 3), "\n")
+  cat("Mean log10(LR):", round(x$meanLogLR, 3), "\n")
   ip = x$IP
-  cat("Estimated inclusion power:", if(!length(ip)) NA, "\n")
+  cat("Estimated power:", if(!length(ip)) NA, "\n")
   for(i in seq_along(ip))
-    cat(sprintf("  P(LR > %s) = %.3g\n", names(ip)[i], ip[i]))
+    cat(sprintf("  P(LR >= %s) = %.3g\n", names(ip)[i], ip[i]))
 }
 
 
