@@ -9,11 +9,13 @@
 #' @param ids A character (or coercible to character) with ID labels indicating
 #'   whose genotypes should be simulated.
 #' @param markers A list of marker objects, or a vector containing names or
-#'   indices referring to markers attached to `x`. By default (`markers = NULL`)
-#'   all attached markers are used. The simulations will be conditional on the
-#'   locus attributes (allele frequencies, mutation models a.s.o.) and any
-#'   existing genotypes in the indicated markers.
+#'   indices referring to markers attached to `x`. By default (`NULL`), all
+#'   attached markers are used. The simulations will be conditional on the locus
+#'   attributes (allele frequencies, mutation models a.s.o.) and any existing
+#'   genotypes in the indicated markers.
 #' @param seed NULL, or a numeric seed for the random number generator.
+#' @param numCores The number of cores used for parallelisation, by default 1.
+#' @param verbose A logical, by default TRUE.
 #' @param ... Further arguments passed on to [markerSim()].
 #'
 #' @return A list of `N` objects similar to `x`, but with simulated genotypes.
@@ -33,9 +35,14 @@
 #' # Simulate 3 profiles of B2 conditional on the above
 #' profileSim(x, N = 3, ids = "B2")
 #'
-#'
+#' @importFrom parallel makeCluster stopCluster detectCores parLapply
+#'   clusterEvalQ clusterExport clusterSetRNGStream
 #' @export
-profileSim = function(x, N = 1, ids = NULL, markers = NULL, seed = NULL, ...){
+profileSim = function(x, N = 1, ids = NULL, markers = NULL, seed = NULL,
+                      numCores = 1, verbose = TRUE, ...){
+
+  if(!is.ped(x) && !is.pedList(x))
+    stop2("The first argument must be a `ped` object or a list of such")
 
   # Set seed once (instead of passing it to markerSim)
   if(!is.null(seed))
@@ -53,7 +60,7 @@ profileSim = function(x, N = 1, ids = NULL, markers = NULL, seed = NULL, ...){
 
     res_compwise = lapply(x, function(comp)
       profileSim(comp, N = N, ids = if(!is.null(ids)) intersect(ids, labels(comp)),
-                 markers = markers, ...))
+                 markers = markers, numCores = numCores, verbose = verbose, ...))
 
     # Transpose: Collect j'th sim of each component.
     res = lapply(1:N, function(j) lapply(res_compwise, `[[`, j))
@@ -80,13 +87,37 @@ profileSim = function(x, N = 1, ids = NULL, markers = NULL, seed = NULL, ...){
     mnames = name(x, markers)
   nonNAs = which(!is.na(mnames))
 
+
   ### SIMULATIONS ###
 
-  # Iterate over the loci, make N simulations of each.
-  sims_markerwise = lapply(markers, function(pm)
-      markerSim(x, N = N, ids = ids, partialmarker = pm, verbose = FALSE, ...))
+  # Parallelise?
+  if(is.na(numCores))
+    numCores = max(detectCores() - 1, 1)
 
-  # Transpose: Extract i'th marker from each sim above.
+  if(numCores > 1) {
+    cl = makeCluster(numCores)
+    on.exit(stopCluster(cl))
+    if(verbose) {
+      message("Preparing parallelisation using ", length(cl), " cores")
+    }
+    clusterEvalQ(cl, library(forrel))
+    clusterExport(cl, c("markerSim", "N", "ids"), envir = environment())
+
+    # Random number seed for cluster. NB: Keep this outside of function call
+    iseed = sample.int(1e6,1)
+    clusterSetRNGStream(cl, iseed = iseed)
+
+    # Iterate over the loci, make N simulations of each.
+    sims_markerwise = parLapply(cl, markers, function(pm)
+      markerSim(x, N = N, ids = ids, partialmarker = pm, verbose = FALSE))
+  }
+  else {
+    # Iterate over the loci, make N simulations of each.
+    sims_markerwise = lapply(markers, function(pm)
+        markerSim(x, N = N, ids = ids, partialmarker = pm, verbose = FALSE, ...))
+  }
+
+  ### Transpose: Extract i'th marker from each sim above.
   # Output: List of N `ped`s, each with length(markers) attached markers
   sims = lapply(1:N, function(i) {
     mlist = lapply(sims_markerwise, function(y) y$MARKERS[[i]])
